<?php

namespace Drupal\if_then_else;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use function GuzzleHttp\json_decode;
use Drupal\if_then_else\core\Conditions\IfthenelseFormAlterCondition;
use Drupal\if_then_else\Event\IfthenelseNodesGenerate;

/**
 * Defines a class to build a ifthenelseRule entity form.
 *
 * @see \Drupal\if_then_else
 */
class IfthenelseRuleForm extends EntityForm {

  /**
   * Constructs an ExampleForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entityTypeManager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    $form = parent::form($form, $form_state);

    // Current entity object.
    $entity = $this->entity;

    // Prepare retejs nodes passed by different modules.
    $ifthenelseNodes = [];

    // Instantiate our event.
    $event = new IfthenelseNodesGenerate($ifthenelseNodes, $entity);

    // Get the event_dispatcher server and dispatch the event.
    $event_dispatcher = \Drupal::service('event_dispatcher');
    $event_dispatcher->dispatch(IfthenelseNodesGenerate::EVENT_NAME, $event);

    // Attach all libraries defined by all modules which are subscribing
    // above event.
    foreach ($event->ifthenelseNodes as $nodes) {
      if (isset($nodes['library'])) {
        $form['#attached']['library'][] = $nodes['library'];
      }

      if (isset($nodes['drupalSettings']) && is_array($nodes['drupalSettings']) && !empty($nodes['drupalSettings'])) {
        $form['#attached']['drupalSettings']['ifthenelserule'] = $nodes['drupalSettings'];
      }
    }

    // Adding retejs intialization library.
    $form['#attached']['library'][] = 'if_then_else/external';

    // Container to create retejs editor and nodes.
    $form['rete-container'] = [
      '#type' => 'item',
      '#markup' => '<div class="rete-container"><div id="rete"></div></div>',
    ];

    if (isset($entity->data)) {
      $form['#attached']['drupalSettings']['ifthenelserule']['data'] = $entity->data;
    }
    else {
      $form['#attached']['drupalSettings']['ifthenelserule']['data'] = FALSE;
    }

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => t('Label'),
      '#required' => TRUE,
      '#default_value' => $entity->label(),
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity->id(),
      '#machine_name' => [
        'exists' => [$this, 'exist'],
      ],
      '#disabled' => !$entity->isNew(),
    ];

    $form['active'] = [
      '#type' => 'checkbox',
      '#title' => t('Active'),
      '#default_value' => isset($entity->active) ? $entity->active : '',
    ];

    $form['module'] = [
      '#type' => 'hidden',
      '#title' => t('Module'),
      '#value' => 'ifthenelse',
      '#default_value' => isset($entity->module) ? $entity->module : '',
    ];

    // @todo: For now the event is hardcoded using javascript to form_alter
    // Need to make that dynamic based on what event node is added on retejs
    $form['event'] = [
      '#type' => 'hidden',
      '#title' => t('Module'),
      '#attributes' => [
        'id' => 'ifthenelse-event',
      ],
      '#default_value' => isset($entity->event) ? $entity->event : '',
    ];

    $form['data'] = [
      '#type' => 'hidden',
      '#title' => t('Module'),
      '#attributes' => [
        'id' => 'ifthenelse-data',
      ],
      '#default_value' => isset($entity->data) ? $entity->data : '',
    ];

    return $form;
  }

  /**
   * Entity form validation function.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $data = json_decode($form_state->getValue('data'));
    $event = $form_state->getValue('event');
    if ($event == 'form_alter') {
      foreach ($data->nodes as $node) {
        if ($node->data->type == 'condition' && $node->data->selected_entity->value == 'other_form') {
          $form_class = $node->data->otherFormClass;
          $ifthenelseFormAlterCondition = new IfthenelseFormAlterCondition([]);
          $valid_form = $ifthenelseFormAlterCondition->validateFormClass($form_class);
          if (!$valid_form) {
            $form_state->setErrorByName('rete-container', t('Form class provided is not an instance of FormInterface.'));
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->entity;
    $status = $entity->save();

    if ($status) {
      $this->messenger()->addMessage($this->t('Saved the %label Example.', [
        '%label' => $entity->label(),
      ]));
    }
    else {
      $this->messenger()->addMessage($this->t('The %label Example was not saved.', [
        '%label' => $entity->label(),
      ]), MessengerInterface::TYPE_ERROR);
    }

    // Redirect to Rule list page.
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
  }

  /**
   * Helper function to check whether an Example configuration entity exists.
   */
  public function exist($id) {
    $entity = $this->entityTypeManager->getStorage('ifthenelserule')->getQuery()
      ->condition('id', $id)
      ->execute();
    return (bool) $entity;
  }

}
