<?php

namespace Drupal\if_then_else\core;

use Drupal\field\Entity\FieldConfig;
use Drupal\Core\Entity\ContentEntityType;
use Drupal\Core\Form\FormInterface;

/**
 * Class defined to have common functions for ifthenelse rules processing.
 */
class IfthenelseUtilities {

  /**
   * Get content entities and bundles.
   *
   * @return array
   *   Return array of Content entities and their bundles
   */
  public function getContentEntitiesAndBundles() {
    $content_entity_types = [];

    // Fetching all entities.
    $entity_type_definitions = \Drupal::entityTypeManager()->getDefinitions();
    $bundle_info = \Drupal::service("entity_type.bundle.info")->getAllBundleInfo();
    /* @var $definition EntityTypeInterface */
    foreach ($entity_type_definitions as $definition) {

      // Checking if the entity is of type content.
      if ($definition instanceof ContentEntityType) {
        $entity_id = $definition->id();
        if ($entity_id == 'file' || $entity_id == 'shortcut' || $entity_id == 'menu_link_content') {
          // Skipping some of the content entities.
          continue;
        }

        $content_entity_types[$entity_id]['entity_id'] = $entity_id;
        $content_entity_types[$entity_id]['label'] = $definition->getLabel()->__toString();
        // Fetching all bundles of entity.
        $entity_bundles = $bundle_info[$entity_id];

        // Getting label of each bundle of an entity.
        foreach ($entity_bundles as $bundle_id => $bundle) {
          if (is_object($bundle['label'])) {
            $content_entity_types[$entity_id]['bundles'][$bundle_id]['label'] = $bundle['label']->__toString();
          }
          elseif (!is_object($bundle['label']) && !is_array($bundle['label'])) {
            $content_entity_types[$entity_id]['bundles'][$bundle_id]['label'] = $bundle['label'];
            $content_entity_types[$entity_id]['bundles'][$bundle_id]['bundle_id'] = $bundle_id;
          }
        }
      }
    }

    return $content_entity_types;
  }

  /**
   * Get list of fields by entity and bundle id.
   *
   * @param array $content_entity_types
   *   List of content entities and bundles.
   *
   * @return array
   *   List of fields associated with bundle.
   */
  public function getFieldsByEntityBundleId(array $content_entity_types) {
    $listFields = [];
    $entity_field_manager = \Drupal::service('entity_field.manager');

    foreach ($content_entity_types as $entity) {
      $entity_id = $entity['entity_id'];
      foreach ($entity['bundles'] as $bundle_id => $bundle) {
        $fields = $entity_field_manager->getFieldDefinitions($entity_id, $bundle_id);
        foreach ($fields as $field_name => $field_definition) {
          if (!empty($field_definition->getTargetBundle())) {
            // List of all fields in an entity bundle.
            $listFields[$field_name]['name'] = $field_definition->getLabel();
            if (is_object($listFields[$field_name]['name'])) {
              $listFields[$field_name]['name'] = $listFields[$field_name]['name']->__toString();
            }
            $listFields[$field_name]['code'] = $field_name;
          }
        }
      }
    }

    // Converting it to non associative array for working with
    // Vuejs multiselect.
    $listFieldsAssoc = $listFields;
    $listFields = [];
    $i = 0;
    foreach ($listFieldsAssoc as $field) {
      $listFields[$i]['name'] = $field['name'];
      $listFields[$i]['code'] = $field['code'];
      $i++;
    }
    return $listFields;
  }

  /**
   * Get a specific field by entity, bundle id and field id.
   *
   * @param string $entity_id
   *   Content Entity id.
   * @param string $bundle_id
   *   Bundle id whose fields to be fetched.
   * @param string $field_name
   *   Field name.
   *
   * @return array
   *   Field definition and info for a specific field.
   */
  public function getFieldInfoByEntityBundleId($entity_id, $bundle_id, $field_name) {

    // Get field definitation.
    $field = FieldConfig::loadByName($entity_id, $bundle_id, $field_name);
    $field_type = $field->getType();

    // List text type of field.
    if ($field_type == 'list_string') {
      $field_settings = $field->getFieldStorageDefinition()->getSettings();
      foreach ($field_settings['allowed_values'] as $key => $value) {
        $field_value[] = [
          'key' => $key,
          'name' => $value,
        ];
      }
      $field_type = 'select-input';
      $field_label = $field->getLabel();
    }

    // Plain text type of field.
    if ($field_type == 'string') {
      $field_value = '';
      $field_type = 'text-input';
      $field_label = $field->getLabel();
    }

    // Plain text type of field.
    if ($field_type == 'text_with_summary') {
      $field_value = '';
      $field_type = 'textarea-input';
      $field_label = $field->getLabel();
    }

    $field_info['type'] = $field_type;
    $field_info['field_name'] = $field_name;
    $field_info['value'] = $field_value;
    $field_info['field_label'] = $field_label;
    return $field_info;
  }

  /**
   * Get form fields by form class.
   *
   * @param string $form_class
   *   Form class name which extends FormInterface class.
   *
   * @return array
   *   List of fields in form.
   */
  public function getFieldsByFormClass($form_class) {
    $listFields = [];
    if (is_string($form_class) && class_exists($form_class)) {
      // Generating class object from class string name to compare
      // if it is instance of FormInterface.
      $other_form_class = \Drupal::classResolver($form_class);
      if (!is_object($other_form_class) || !($other_form_class instanceof FormInterface)) {
        // @todo
        // exception if the form class entered is wrong.
      }
      else {
        $other_form = \Drupal::formBuilder()->getForm($form_class);

        $i = 0;
        // Iterate all keys of form array.
        foreach ($other_form as $key => $field) {
          // Skip all keys which starts with #. they are not fields.
          if (strpos($key, '#') === FALSE) {
            if ($field['#type'] == 'hidden' || $field['#type'] == 'token' || $field['#type'] == 'actions' || $field['#type'] == 'details' ||$field['#type'] == 'vertical_tabs') {
              // Skip all keys which can't be made required.
              continue;
            }

            if ($field['#type'] == 'container') {
              if (isset($field['widget'])) {
                foreach ($field['widget'] as $k => $value) {
                  if (strpos($k, '#') !== FALSE) {
                    // Skip all keys which have #.
                    continue;
                  }

                  // If title is translatable object.
                  if (is_object($field['widget'][$k]['#title'])) {
                    $listFields[$i]['name'] = $field['widget'][$k]['#title']->__toString();
                  }
                  elseif (is_string($field['widget'][$k]['#title'])) {
                    $listFields[$i]['name'] = $field['widget'][$k]['#title'];
                  }
                  $listFields[$i]['code'] = $key;
                  $i++;
                }
              }
            }
            else {
              if (is_object($field['#title'])) {
                $listFields[$i]['name'] = $field['#title']->__toString();
              }
              elseif (is_string($field['#title'])) {
                $listFields[$i]['name'] = $field['#title'];
              }
              $listFields[$i]['code'] = $key;
              $i++;
            }
          }
        }
      }
    }

    return $listFields;
  }

}
